@@ -1,7 +1,7 @@
 /*
  *  spinbox2.cpp  -  spin box with extra pair of spin buttons
  *  Program:  kalarm
- *  Copyright © 2001-2019 David Jarvie <djarvie@kde.org>
+ *  Copyright © 2001-2020 David Jarvie <djarvie@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,11 +30,8 @@
 #include <QTimer>
 #include <QFrame>
 #include <QBrush>
-#include <QStyle>
-#include <QObject>
 #include <QApplication>
 #include <QPixmap>
-#include <QMatrix>
 
 #include <stdlib.h>
 
@@ -549,9 +546,9 @@ void SpinMirror::setMirroredState(bool clear)
 {
     // Some styles only look right when the buttons are mirrored
     if (mMirrored)
-        setMatrix(QMatrix(-1, 0, 0, 1, width() - 1 , 0));  // mirror left to right
+        setTransform(QTransform(-1, 0, 0, 1, width() - 1 , 0));  // mirror left to right
     else if (clear)
-        setMatrix(QMatrix());
+        setTransform(QTransform());
 }
 
 void SpinMirror::setFrame()
@@ -661,12 +658,12 @@ void SpinMirror::mouseEvent(QMouseEvent* e)
 {
     if (mReadOnly)
         return;
-    QPoint pt = e->pos();
+    QPointF pt = e->pos();
     QGraphicsItem* item = scene()->itemAt(pt, QTransform());
     if (item == mButtons)
         pt = spinboxPoint(pt);
     else
-        pt = QPoint(0, 0);  // allow auto-repeat to stop
+        pt = QPointF(0, 0);  // allow auto-repeat to stop
     QApplication::postEvent(mSpinbox, new QMouseEvent(e->type(), pt, e->button(), e->buttons(), e->modifiers()));
 }
 
@@ -678,23 +675,31 @@ void SpinMirror::wheelEvent(QWheelEvent* e)
 {
     if (mReadOnly)
         return;
-    QPoint pt = e->pos();
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+    QPointF pt = e->posF();
+#else
+    QPointF pt = e->position();
+#endif
     QGraphicsItem* item = scene()->itemAt(pt, QTransform());
     if (item == mButtons)
     {
         pt = spinboxPoint(pt);
-        QApplication::postEvent(mSpinbox, new QWheelEvent(pt, e->delta(), e->buttons(), e->modifiers()), e->orientation());
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+        QApplication::postEvent(mSpinbox, new QWheelEvent(pt, e->globalPosF(), e->pixelDelta(), e->angleDelta(), e->buttons(), e->modifiers(), e->phase(), e->inverted(), e->source()));
+#else
+        QApplication::postEvent(mSpinbox, new QWheelEvent(pt, e->globalPosition(), e->pixelDelta(), e->angleDelta(), e->buttons(), e->modifiers(), e->phase(), e->inverted(), e->source()));
+#endif
     }
 }
 
 /******************************************************************************
 * Translate SpinMirror coordinates to those of the mirrored spinbox.
 */
-QPoint SpinMirror::spinboxPoint(const QPoint& param) const
+QPointF SpinMirror::spinboxPoint(const QPointF& param) const
 {
     const QRect r = mSpinbox->upRect();
     const QPointF ptf = mButtons->mapFromScene(param.x(), param.y());
-    QPoint pt(ptf.x(), ptf.y());
+    QPointF pt(ptf.x(), ptf.y());
     pt.setX(ptf.x() + r.left());
     pt.setY(ptf.y() + r.top());
     return pt;
@@ -740,7 +745,7 @@ bool SpinMirror::event(QEvent* e)
 
     if (he)
     {
-        QApplication::postEvent(mSpinbox, new QHoverEvent(e->type(), spinboxPoint(he->pos()), spinboxPoint(he->oldPos())));
+        QApplication::postEvent(mSpinbox, new QHoverEvent(e->type(), spinboxPoint(he->posF()), spinboxPoint(he->oldPosF())));
         setButtons();
     }
 
